#ifndef STATUSWIDGET_H
#define STATUSWIDGET_H

#include <QWidget>
#include <QSettings>
#include <QTime>
#include <QMap>
#include <QPropertyAnimation>
#include <QGraphicsOpacityEffect>
#include <QLabel>

#include "models/patchlist.h"

namespace Ui {
class StatusWidget;
}

class StatusWidget : public QWidget
{
    Q_OBJECT

public:
    // States. Higher state value means higher priority on visualisation
    enum ToolTipState : int {
        E_INFO = 4,
        E_ERROR = 2,
        E_PROCESS = 1,
        E_RANDOM = 0 // (state | E_RANDOM) should be always true (we can always show random message)
    };

    explicit StatusWidget(PatchList *legacy_patches, QWidget *parent = 0);
    ~StatusWidget();

public slots:
    void updateFontsSizes();

    void setToolTipMessage(QString message, ToolTipState state);

    void unsetToolTipMessage(ToolTipState state);

protected:
    void resizeEvent(QResizeEvent *event) override;

private slots:
    void fadeBetweenToolTips(QString tooltip_message);

    void onPatchTotalOperationsStarted();
    void onPatchTotalOperationsFinished();

    void onPatchOperationStarted(Patch::Operation operation, Patch* patch);
    void onPatchOperationFinished(Patch::Operation operation, Patch* patch);

    void onPatchTotalProgressChanged(Patch::OperationProgress operation_progress);
    void onPatchProgressChanged(Patch::OperationProgress progress, Patch* patch);

    void on_game_button_clicked();

    void on_check_for_updates_button_clicked();

private:
    void updateStatusBar(Patch::OperationProgress progress);

private:
    Ui::StatusWidget *ui;

    PatchList *legacy_patches_;

    bool all_patch_operations_finished_ = false;

    QMap<QString, QGraphicsOpacityEffect*> tooltip_effects;
    QMap<QString, QPropertyAnimation*> tooltip_animations;

    bool galadriel_tooltip_1_type = false; // false = hiding, true - showing. Tooltip 2 is always opposite to tooltip 1

    QMap<Patch*, Patch::Operation> patch_operations;

    QTime last_statusbar_update_time_;

    ToolTipState tooltip_state_ = E_RANDOM;
    QMap<ToolTipState, QString> tooltip_messages_;

};

#endif // STATUSWIDGET_H
